package de.ozgcloud.document.bescheid;

import java.util.Optional;
import java.util.stream.Stream;

import de.ozgcloud.document.bescheid.BescheidServiceGrpc.BescheidServiceImplBase;
import de.ozgcloud.document.bescheid.vorgang.VorgangId;
import io.grpc.stub.StreamObserver;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
@RequiredArgsConstructor
class BescheidGrpcService extends BescheidServiceImplBase {

	private final BescheidService service;
	private final BescheidMapper mapper;

	@Override
	public void getBescheidDraft(GrpcGetBescheidDraftRequest request, StreamObserver<GrpcGetBescheidDraftResponse> responseObserver) {
		var bescheidDraft = service.findDraft(request.getVorgangId());

		responseObserver.onNext(buildGetBescheidDraftResponse(bescheidDraft));
		responseObserver.onCompleted();
	}

	private GrpcGetBescheidDraftResponse buildGetBescheidDraftResponse(Optional<Bescheid> bescheidDraft) {
		return bescheidDraft.map(mapper::toGetBescheidDraftResponse).orElseGet(GrpcGetBescheidDraftResponse::getDefaultInstance);
	}

	@Override
	public void getConfig(GrpcBescheidManagerConfigRequest request, StreamObserver<GrpcBescheidManagerConfigResponse> responseObserver) {
		responseObserver.onNext(mapper.fromBescheidManagerConfig(service.getConfig()));
		responseObserver.onCompleted();
	}

	@Override
	public void getAll(GrpcGetAllBescheidRequest request, StreamObserver<GrpcGetAllBescheidResponse> responseObserver) {
		var bescheide = service.findAll(VorgangId.from(request.getVorgangId()));

		responseObserver.onNext(buildGetAllBescheidResponse(bescheide));
		responseObserver.onCompleted();
	}

	GrpcGetAllBescheidResponse buildGetAllBescheidResponse(Stream<Bescheid> bescheidItems) {
		return GrpcGetAllBescheidResponse.newBuilder().addAllBescheid(bescheidItems.map(mapper::toBescheid).toList()).build();
	}

	@Override
	public void getBescheid(GrpcGetBescheidRequest request, StreamObserver<GrpcGetBescheidResponse> responseObserver) {
		var bescheid = service.getBescheid(request.getId());

		responseObserver.onNext(mapper.toGetBescheidResponse(bescheid));
		responseObserver.onCompleted();
	}
}