package de.ozgcloud.document.bescheid.common.freemarker;

import static org.assertj.core.api.Assertions.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;

import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import de.ozgcloud.apilib.common.command.grpc.CommandMapper;
import de.ozgcloud.apilib.user.UserProfileMapper;
import de.ozgcloud.common.test.ITCase;
import de.ozgcloud.document.bescheid.BescheidService;

@ITCase
class TemplateHandlerITCase {

	private static final String BESCHEID_TEMPLATE = "dummy.template.txt.ftlh";

	@Autowired
	private TemplateHandler handler;
	@MockBean
	private OzgCloudCommandService commandService;
	@MockBean
	private CommandMapper commandMapper;
	@MockBean
	private UserProfileMapper userProfileMapper;

	@DisplayName("Get raw template")
	@Nested
	class TestGetRawTemplate {

		@Test
		void shouldReturnTemplateAsString() {
			var content = handler.getRawTemplate(BESCHEID_TEMPLATE);

			assertThat(content)
					.contains("Sehr geehrte/r Antragsteller/in")
					.contains("im folgenden wird evtl was ersetzt")
					.contains("valueToReplace")
					.contains("Mit freundlichen Grüßen");
		}
	}

	@DisplayName("Fill template")
	@Nested
	class TestFillTemplate {

		@Test
		void shouldReturnFilledTemplate() {
			var signatur = "ganz andere Signatur";
			var content = handler.fillTemplate(BescheidService.DEFAULT_NACHRICHT_TEMPLATE_FILE,
					Map.of(BescheidService.KEY_TEMPLATE_SIGNATURE, signatur));

			assertThat(content)
					.contains("Sehr geehrte/r Antragsteller/in")
					.contains("im Folgenden erhalten Sie Ihren Bescheid.")
					.contains(signatur);
		}

		@Test
		void shouldReturnWithDefaultValue() {
			var content = handler.fillTemplate(BescheidService.DEFAULT_NACHRICHT_TEMPLATE_FILE, Collections.emptyMap());

			assertThat(content)
					.contains("Sehr geehrte/r Antragsteller/in")
					.contains("im Folgenden erhalten Sie Ihren Bescheid.")
					.contains("Mit freundlichen Grüßen")
					.contains("Ihre Verwaltung");
		}
	}
}
